/**
 * Module dependencies.
 */

var express = require('express');
var routes = require('./routes');
var http = require('http');
var path = require('path');
//var favicon = require('serve-favicon');
var logger = require('morgan');
var methodOverride = require('method-override');
//var session = require('express-session');
var bodyParser = require('body-parser');
var multer = require('multer');
var errorHandler = require('errorhandler');

var app = express();

// commandline option
// multiuser mode
var isMultiuser = (process.argv.indexOf("multiuser") !== -1)? true: false;
var isAllowRemote = (process.argv.indexOf("allowremote") !== -1)? true: false;
app.set('isMultiuser', isMultiuser);
app.set('isAllowRemote', isAllowRemote);

// all environments
app.set('port', process.env.PORT || 3001);
app.set('views', __dirname + '/views');
app.set('view engine', 'pug');
//app.use(favicon());
app.use(logger('dev'));
//app.use(bodyParser({uploadDir: path.join(__dirname, 'tmp')}));
app.use(methodOverride());
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({limit:'50mb', extended: true, parameterLimit: 1000000}));
//app.use(app.router);
app.use(express.static(path.join(__dirname, 'public')));

// development only
if ('development' == app.get('env')) {
  app.use(errorHandler());
}

// access restriction
app.use(function(req, res, next){
    var hostname = req.headers.host;
    hostname = hostname.split(":")[0];
    if(hostname  === null || hostname === undefined){
        res.send(400);
        return;
    }
    // if the request is from localhost, accept.
    if(hostname === "localhost" || hostname === "127.0.0.1"){
        next();
        return;
    }
    // if the "isAllowRemote" is true or multiuser mode, accept remote access.
    else if(isAllowRemote || isMultiuser){
        next();
        return;
    }
    // if the remote access is not allowed, respond 400.
    else{
        res.send(400);
    }
});

// RESTful API
app.post('/tfgen/:mode', routes.tfgen);

app.get('/file/:filename', routes.download);
app.post('/file/:filetype', multer({dest: path.join(__dirname, 'tmp')}).single('file'), routes.upload);

app.post('/valid/:filetype', routes.valid);
app.post('/genposxml', routes.genposxml);
app.post('/updposxml', routes.updposxml);
app.post('/genconfig', routes.genconfig);
app.post('/mergexmls', routes.mergexmls);
app.post('/mergesrcxmls', routes.mergesrcxmls);
app.post('/genneighbors', routes.genneighbors);
app.post('/gentf', routes.gentf);

app.post('/genzip2AMPcoord', routes.genzip2AMPcoord);
app.post('/genzip2DBcoord', routes.genzip2DBcoord);
app.post('/genzip2PHASEcoord', routes.genzip2PHASEcoord);
app.post('/genzip2IFFTcoord', routes.genzip2IFFTcoord);

app.get('/progress', routes.progress);
app.get('/utilpath', routes.utilpath);

app.get("/", routes.index);

// default rule
app.get(/^\/(tspgen|tfgen|evaluate|tfplot|micplot|srcplot|micgen|srcgen|tfgen)/, routes.harktool);

http.createServer(app).listen(app.get('port'), function(){
  console.log('Express server listening on port ' + app.get('port'));
});
